
/*********************************************************************/
/* File:   MyMaxwellAssemble_small.cpp                    */
/* Author: Julia Hauser                                   */
/* Date:   3. 1. 2019                                     */
/*********************************************************************/


#include <solve.hpp>
using namespace ngsolve;
#include "utility_functions.hpp"


namespace mymaxassembleSMALL
{
	
	template<int Dy>
	void CalcElementMatrix_tpy(const FiniteElement & fely,
							const ElementTransformation &eltrans_y, LocalHeap & lhy, 
							FlatMatrix<double> elmat_fesy,FlatMatrix<double> elmatD_fesy)
	{
        const HCurlFiniteElement<Dy>  & fely_cast = 
                        dynamic_cast<const HCurlFiniteElement<Dy> &>(fely);
                    
        ELEMENT_TYPE eltypey = fely_cast.ElementType();

        int ndy = fely_cast.GetNDof();

        MatrixFixWidth<Dy,double> shapey(ndy);
        FlatMatrix<> dshapey(ndy,(Dy == 2)? 1:3,lhy);


        const IntegrationRule & ir_voly =
                        SelectIntegrationRule (eltypey, 2*fely_cast.Order());

        FlatMatrix<double> elmaty(ndy, lhy);
        FlatMatrix<double> elmatDy(ndy, lhy);
        elmaty = 0.0;
        elmatDy = 0.0;

        for (int l = 0; l < ir_voly.Size(); l++)
        {
            HeapReset hr(lhy);
            IntegrationPoint & ip = ir_voly[l];
            const MappedIntegrationPoint<Dy,Dy> sip(ip, eltrans_y);
            BaseMappedIntegrationPoint & mip = eltrans_y(ip, lhy);
            auto conv = 1.0;

            fely_cast.CalcMappedShape (mip, shapey);
            fely_cast.CalcMappedCurlShape (mip, dshapey);

            elmaty  += ip.Weight() *sip.GetMeasure()*  shapey * Trans (shapey);
            elmatDy += ip.Weight() *sip.GetMeasure()* (conv * dshapey) * Trans (dshapey);
        }

        elmat_fesy = 0.0;
        elmatD_fesy = 0.0;
        for(int jind = 0; jind<ndy;jind++)
        {
                for(int lind = 0; lind<ndy;lind++)
                {
                    //cout<<elmaty(jind, lind)<< "  ";
                        elmat_fesy(jind,lind) = elmaty(jind, lind);
                        elmatD_fesy(jind,lind) = elmatDy(jind, lind);
                }
                //cout<<endl;
        }
        //cout<<endl;
	}


    void MyMaxwellAssemble_small(shared_ptr<FESpace> fes_in,
                            shared_ptr<CoefficientFunction> eps_coef,
                            shared_ptr<CoefficientFunction> mu_coef,
                            shared_ptr<CoefficientFunction> f_coef)
    {
        TPHighOrderFESpace * fes = dynamic_cast<TPHighOrderFESpace *>(fes_in.get());
        cout << "We assemble matrix and rhs vector" << endl;
        cout << "For fes=(fes_t,fes_x)" << endl;

        const shared_ptr<FESpace> & fes_y = fes->Space(0);

        auto ma_y = fes_y->GetMeshAccess();

        int ndof_y = fes_y->GetNDof();
        int ne_y = ma_y->GetNE();
        
        
        
        Array<int> dnums;
        Array<int> cnty(ne_y);

        for (auto eiy : ma_y->Elements(VOL))
        {
            fes_y->GetDofNrs (eiy, dnums);
            cnty[eiy.Nr()] = dnums.Size();
        }
        
        Table<int> ely2dof(cnty);
        Array<int> dnumsy;
        
        for (auto eiy : ma_y->Elements(VOL))
        {
            fes_y->GetDofNrs (eiy, dnumsy);
            ely2dof[eiy.Nr()] = dnumsy;
        }
        cout<<"Start filling matrix "<<endl;
        
        LocalHeap lhy(1000000);
        
        auto maty = make_shared<SparseMatrixSymmetric<double>> (ndof_y, ely2dof);
        auto matDy = make_shared<SparseMatrixSymmetric<double>> (ndof_y, ely2dof);
    
        IterateElements(*fes_y, VOL, lhy, [&] (FESpace::Element ely, LocalHeap &lhy)
        {
            const ElementTransformation& eltrans_y = ma_y->GetTrafo(ely,lhy);
            const FiniteElement& fely = fes_y->GetFE(ely,lhy);
            FlatArray<DofId> dofsy = ely.GetDofs();
            
            FlatMatrix<> elmat_fesy(dofsy.Size(),lhy); 
            FlatMatrix<> elmatD_fesy(dofsy.Size(),lhy); 
            
            CalcElementMatrix_tpy<2>(fely,eltrans_y,lhy,elmat_fesy,elmatD_fesy);
            maty->AddElementMatrixSymmetric(dofsy,elmat_fesy);
            matDy->AddElementMatrixSymmetric(dofsy,elmatD_fesy);
        });
                    
    cout <<"mat_y = "<<*maty<<endl;
    //*testout <<"matD_y = "<<*matDy<<endl;
    }
}
